@@ -1,5 +1,7 @@
 `segmented.lm` <-
 function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = TRUE, ...) {
+#preliminary version that should account the bootstrap restart
+#o<-segmented.lm1(lm(y~x),seg.Z=~x,psi=.2,control=seg.control(display=FALSE,n.boot=20,it.max=5))
     n.Seg<-1
     if(is.list(psi)){
       if(length(all.vars(seg.Z))!=length(psi)) stop("A wrong number of terms in `seg.Z' or `psi'")
@@ -11,12 +13,18 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     toll <- control$toll
     visual <- control$visual
     stop.if.error<-control$stop.if.error
-###    if() stop.if.error<-FALSE
+    n.boot<-control$n.boot
+    size.boot<-control$size.boot
+    gap<-control$gap
+    if(n.boot>0){
+        if(visual) {visual<-FALSE; warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
+        if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
+     }
     last <- control$last
     K<-control$K
     h<-min(abs(control$h),1)
     if(h<1) it.max<-it.max+round(it.max/2)
-#    if(!stop.if.error) objInitial<-obj        
+#    if(!stop.if.error) objInitial<-obj
     #-------------------------------
 #    #una migliore soluzione.........
 #    objframe <- update(obj, model = TRUE, x = TRUE, y = TRUE)
@@ -30,7 +38,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
     if(class(mf$formula)=="name" && !"~"%in%paste(mf$formula)) mf$formula<-eval(mf$formula)
-    orig.call$formula<-update.formula(orig.call$formula, paste("~.-",all.vars(seg.Z))) #utile per plotting
+    #orig.call$formula<-update.formula(orig.call$formula, paste("~.-",all.vars(seg.Z)))
     if(length(all.vars(formula(obj)))>1){
     mf$formula<-update.formula(mf$formula,paste(paste(seg.Z,collapse=".+"),"+",paste(all.vars(formula(obj))[-1],collapse="+")))
     } else {
@@ -41,7 +49,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     #       altrimenti nel model.frame non risulta l'offset
     weights <- as.vector(model.weights(mf))
     offs <- as.vector(model.offset(mf))
-    
+
     if(!is.null(Call$weights)){ #"(weights)"%in%names(mf)
       names(mf)[which(names(mf)=="(weights)")]<-as.character(Call$weights)
       # mf["(weights)"]<-weights
@@ -61,7 +69,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
         XREG<-cbind(XREG,new.XREGseg)
         }
     n.psi<- length(unlist(psi))
-    id.n.Seg<-(ncol(XREG)-n.Seg+1):ncol(XREG)    
+    id.n.Seg<-(ncol(XREG)-n.Seg+1):ncol(XREG)
     XREGseg<-XREG[,id.n.Seg,drop=FALSE]
     #XREG<-XREG[,-id.n.Seg,drop=FALSE]
     #XREG<-model.matrix(obj0) non va bene perch non elimina gli eventuali mancanti in seg.Z..
@@ -80,7 +88,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
         }
     if (!is.list(Z) || !is.list(psi) || is.null(names(Z)) || is.null(names(psi))) stop("Z and psi have to be *named* list")
     id.nomiZpsi <- match(names(Z), names(psi))
-    if ((length(Z)!=length(psi)) || any(is.na(id.nomiZpsi))) 
+    if ((length(Z)!=length(psi)) || any(is.na(id.nomiZpsi)))
         stop("Length or names of Z and psi do not match")
     #dd <- match(names(Z), names(psi))
     nome <- names(psi)[id.nomiZpsi]
@@ -88,9 +96,9 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     for(i in 1:length(psi)) {
         if(any(is.na(psi[[i]]))) psi[[i]]<-quantile(Z[[i]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)
         }
-    
+
     a <- sapply(psi, length)
-    
+
     #per evitare che durante il processo iterativo i psi non siano ordinati
     id.psi.group <- rep(1:length(a), times = a) #identificativo di apparteneza alla variabile
     #
@@ -104,6 +112,10 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     psi<-unlist(tapply(psi,id.psi.group,sort))
     k <- ncol(Z)
     PSI <- matrix(rep(psi, rep(n, k)), ncol = k)
+    #controllo se psi  ammissibile..
+    c1 <- apply((Z <= PSI), 2, all) #prima era solo <
+    c2 <- apply((Z >= PSI), 2, all) #prima era solo >
+    if(sum(c1 + c2) != 0 || is.na(sum(c1 + c2))) stop("psi out of the range")
     colnames(Z) <- nomiZ <- rep(nome, times = a)
     ripetizioni <- as.numeric(unlist(sapply(table(nomiZ)[order(unique(nomiZ))], function(xxx) {1:xxx})))
     nomiU <- paste("U", ripetizioni, sep = "")
@@ -113,7 +125,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     #forse non serve crearsi l'ambiente KK, usa mf..
     #obj <- update(obj, formula = Fo, data = mf)
     #if (model.frame) obj$model <- mf
-  #controlla che model.frame() funzioni sull'oggetto restituito    
+  #controlla che model.frame() funzioni sull'oggetto restituito
 #    KK <- new.env()
 #    for (i in 1:ncol(objframe$model)) assign(names(objframe$model[i]), objframe$model[[i]], envir = KK)
     if (it.max == 0) {
@@ -124,14 +136,15 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
         # necessario il for? puoi usare colnames(U)<-nomiU;mf[nomiU]<-U
         for(i in 1:ncol(U)) mf[nomiU[i]]<-U[,i]
         Fo <- update.formula(formula(obj), as.formula(paste(".~.+", paste(nomiU, collapse = "+"))))
-        obj <- update(obj, formula = Fo, data = mf, eval=FALSE)
+        obj <- update(obj, formula = Fo, data = mf, evaluate=FALSE)
         if(!is.null(obj[["subset"]])) obj[["subset"]]<-NULL
         obj<-eval(obj, envir=mf)
         if (model) obj$model <-mf  #obj$model <- data.frame(as.list(KK))
+        names(psi)<-paste(paste("psi", ripetizioni, sep = ""), nomiZ, sep=".")
         obj$psi <- psi
         return(obj)
     }
-    #XREG <- model.matrix(obj) creata sopra         
+    #XREG <- model.matrix(obj) creata sopra
     #o <- model.offset(objframe)
     #w <- model.weights(objframe)
     if (is.null(weights)) weights <- rep(1, n)
@@ -143,8 +156,12 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
 #    psi.values <- NULL
     nomiOK<-nomiU
     opz<-list(toll=toll,h=h,stop.if.error=stop.if.error,dev0=dev0,visual=visual,it.max=it.max,
-        nomiOK=nomiOK,id.psi.group=id.psi.group)
+        nomiOK=nomiOK,id.psi.group=id.psi.group,gap=gap)
+    if(n.boot<=0){
     obj<-seg.lm.fit(y,XREG,Z,PSI,weights,offs,opz)
+    } else {
+    obj<-seg.lm.fit.boot(y, XREG, Z, PSI, weights, offs, opz, n.boot=n.boot, size.boot=size.boot)
+      }
     if(!is.list(obj)){
         warning("No breakpoint estimated", call. = FALSE)
         return(obj0)
@@ -152,16 +169,16 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     nomiOK<-obj$nomiOK
     it<-obj$it
     psi<-obj$psi
-    psi.values<-obj$psi.values
+    psi.values<-if(n.boot<=0) obj$psi.values else obj$boot.restart
     U<-obj$U
     V<-obj$V
-    rangeZ<-obj$rangeZ 
+    rangeZ<-obj$rangeZ
     obj<-obj$obj
     k<-length(psi)
     beta.c<-if(k == 1) coef(obj)["U"] else coef(obj)[paste("U", 1:ncol(U), sep = "")]
     psi.values[[length(psi.values) + 1]] <- psi
     id.warn <- FALSE
-    if (it > it.max) {
+    if (n.boot<=0 && it > it.max) {
         warning("max number of iterations attained", call. = FALSE)
         id.warn <- TRUE
     }
@@ -180,20 +197,27 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
 #        assign(nomiVxb[i], Vxb[, i], envir = KK)
 #    }
     nnomi <- c(nomiU, nomiVxb)
-    Fo <- update.formula(formula(obj0), as.formula(paste(".~.+", 
+    Fo <- update.formula(formula(obj0), as.formula(paste(".~.+",
         paste(nnomi, collapse = "+"))))
     #objF <- update(obj0, formula = Fo, data = KK)
     objF <- update(obj0, formula = Fo, data = mf, evaluate=FALSE)
     if(!is.null(objF[["subset"]])) objF[["subset"]]<-NULL
     objF<-eval(objF, envir=mf)
+    if(!gap){
+        names.coef<-names(objF$coefficients)
+        objF$coefficients<-obj$coefficients
+        names(objF$coefficients)<-names.coef
+        objF$fitted.values<-obj$fitted.values
+        objF$residuals<-obj$residuals
+        }
     if(any(is.na(objF$coefficients))){
     stop("some estimate is NA: premature stopping with a large number of breakpoints?",
-      call. = FALSE)    
+      call. = FALSE)
       }
     Cov <- vcov(objF)
     id <- match(nomiVxb, names(coef(objF)))
     vv <- if (length(id) == 1) Cov[id, id] else diag(Cov[id, id])
-    if(length(initial)!=length(psi)) initial<-rep(NA,length(psi)) 
+    if(length(initial)!=length(psi)) initial<-rep(NA,length(psi))
     psi <- cbind(initial, psi, sqrt(vv))
     rownames(psi) <- colnames(Cov)[id]
     colnames(psi) <- c("Initial", "Est.", "St.Err")
@@ -211,7 +235,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     class(objF) <- c("segmented", class(obj0))
     list.obj[[length(list.obj) + 1]] <- objF
     class(list.obj) <- "segmented"
-    if (last) 
+    if (last)
         list.obj <- list.obj[[length(list.obj)]]
     return(list.obj)
 }
